﻿using UnityEngine;
using System;
using System.IO;
using System.Collections.Generic;
using HarmonyLib;
using Newtonsoft.Json;

public class UIPlus : Mod
{
    static readonly string MOD_NAME = "UIPlus";
    static readonly string MOD_NAMESPACE = "com.KingBR." + MOD_NAME;
    static readonly string prefix = "[" + MOD_NAME + "]: ";
    public static string worldLabelDataPath
    {
        get
        {
            string labelDataFile = MOD_NAME + "_labeldata.json";
            string date = new DateTime(SaveAndLoad.WorldToLoad.lastPlayedDateTicks).ToString(SaveAndLoad.dateTimeFormattingSaveFile);
            string text = Path.Combine(SaveAndLoad.WorldPath, SaveAndLoad.WorldToLoad.name, date);
            if (Directory.Exists(text))
                return Path.Combine(text, labelDataFile);
            return Path.Combine(SaveAndLoad.WorldPath, SaveAndLoad.WorldToLoad.name, date + SaveAndLoad.latestStringNameEnding, labelDataFile);
        }
    }

    static Harmony harmony;


    static Dictionary<uint, string> Labels = new Dictionary<uint, string>();
    static bool labelsLoaded = false;

    public void Start()
    {
        harmony = new Harmony(MOD_NAMESPACE);
        harmony.PatchAll();
        Debug.Log(prefix + "Mod has been loaded!");
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll(harmony.Id);
        labelsLoaded = false;
        Debug.Log(prefix + "Mod has been unloaded!");
    }

    public override void WorldEvent_WorldUnloaded()
    {
        Labels = new Dictionary<uint, string>();
        labelsLoaded = false;
    }
    public override void WorldEvent_WorldLoaded()
    {
        LoadLabels();
    }

    public static void LoadLabels()
    {
        if (!LoadSceneManager.IsGameSceneLoaded)
        {
            Debug.Log(prefix + "Unable to load storage labels as you aren't in a game");
            return;
        }

        try
        {
            if (!File.Exists(worldLabelDataPath))
            {
                Labels = new Dictionary<uint, string>();
                return;
            }

            string tmpStr = File.ReadAllText(worldLabelDataPath);

            Debug.Log(prefix + "pre de-serialize " + tmpStr);
            Labels = JsonConvert.DeserializeObject<Dictionary<uint, string>>(tmpStr) ?? throw new Exception("De-serialization of labels failed");
            labelsLoaded = true;
        }
        catch (Exception _e)
        {
            Debug.LogError(prefix + "Failed to load labels: " + _e.Message);
        }
    }

    public static void SaveLabels() => SaveLabels(worldLabelDataPath);

    public static void SaveLabels(string path)
    {
        if (!LoadSceneManager.IsGameSceneLoaded)
        {
            Debug.Log(prefix + "Unable to save storage labels as you aren't in a game");
            return;
        }

        try
        {
            string tmpStr = JsonConvert.SerializeObject(Labels);

            File.WriteAllText(path, tmpStr);
        }
        catch (System.Exception _e)
        {
            Debug.LogError(prefix + "Failed to save labels: " + _e.Message);
        }
    }

    public void ExtraSettingsAPI_Load() { }
    public static bool ExtraSettingsAPI_Loaded;

    public void ExtraSettingsAPI_ButtonPress(string name)
    {
        if (!ExtraSettingsAPI_Loaded || !LoadSceneManager.IsGameSceneLoaded) return;
        switch (name)
        {
            case "putlabel":
                if (Patch_Storage_Small.storageInstance != null && Labels.ContainsKey(Patch_Storage_Small.storageInstance.ObjectIndex))
                {
                    if (ExtraSettingsAPI_GetInputValue("labelname").IsNullOrEmpty()) break;
                    Debug.Log(LabelCmd(new string[] { ExtraSettingsAPI_GetInputValue("labelname") }));
                }
                break;
            case "removelabel":
                if (Patch_Storage_Small.storageInstance != null && Labels.ContainsKey(Patch_Storage_Small.storageInstance.ObjectIndex))
                {
                    Debug.Log(LabelCmd(new string[] { }));
                }
                break;
            case "deletelabelsWorld":
                Debug.Log(DeleteAllLabels(new string[] { }));
                break;
        }
    }

    public static string ExtraSettingsAPI_GetInputValue(string SettingName) => "";

    [ConsoleCommand(name: "label", docs: "Syntax: label [label name] - Label the storage you are currently looking, if used without args it will remove the current label")]
    public static string LabelCmd(string[] args)
    {
        if (Patch_Storage_Small.storageInstance == null)
        {
            Debug.Log(Patch_Storage_Small.storageInstance);
            return prefix + "You arent looking at any storage right now";
        }

        if (args.Length == 0)
        {
            if (Labels.ContainsKey(Patch_Storage_Small.storageInstance.ObjectIndex)) Labels.Remove(Patch_Storage_Small.storageInstance.ObjectIndex);
            return prefix + "Label removed";
        }

        if (Labels.ContainsKey(Patch_Storage_Small.storageInstance.ObjectIndex))
        {
            Labels[Patch_Storage_Small.storageInstance.ObjectIndex] = string.Join(" ", args);
        }
        else
        {
            Labels.Add(Patch_Storage_Small.storageInstance.ObjectIndex, string.Join(" ", args));
        }
        SaveLabels();
        return prefix + "Label '" + string.Join(" ", args) + "' added";
    }

    [ConsoleCommand(name: "deletelabels", docs: "Syntax: deletelabels - Delete all labels from all world")]
    public static string DeleteAllLabels(string[] args)
    {
        Labels = new Dictionary<uint, string>();
        SaveLabels();
        return prefix + "Deleted all labels from the world";
    }

    [HarmonyPatch(typeof(Storage_Small))]
    internal class Patch_Storage_Small
    {
        [HarmonyPatch("OnFinishedPlacement")]
        [HarmonyPostfix]
        static void OnFinishedPlacement(Storage_Small __instance) => Labels.Add(__instance.ObjectIndex, "");

        [HarmonyPatch("OnDestroy")]
        [HarmonyPostfix]
        static void OnDestroy(Storage_Small __instance)
        {
            if (Labels.ContainsKey(__instance.ObjectIndex))
                Labels.Remove(__instance.ObjectIndex);
        }

        public static Storage_Small storageInstance = null;

        [HarmonyPatch("OnIsRayed")]
        [HarmonyPostfix]
        static void OnIsRayed(Storage_Small __instance, CanvasHelper ___canvas)
        {

            if (CanvasHelper.ActiveMenu == MenuType.None && !PlayerItemManager.IsBusy && ___canvas.CanOpenMenu && Helper.LocalPlayerIsWithinDistance(__instance.transform.position, Player.UseDistance + 0.5f))
            {
                if (!labelsLoaded && (Labels.Keys.Count == 0 || Labels.Values.Join(null, "").IsNullOrEmpty())) LoadLabels();

                storageInstance = __instance;
                if (!Labels.ContainsKey(__instance.ObjectIndex))
                    Labels.Add(__instance.ObjectIndex, "");
            }
        }

        [HarmonyPatch("OnRayExit")]
        [HarmonyPostfix]
        static void OnRayExit()
        {
            storageInstance = null;
        }
    }

    [HarmonyPatch(typeof(Tank))]
    internal class Patch_Tank
    {
        public static Tank tankInstance;

        [HarmonyPatch("OnRayEnter")]
        [HarmonyPostfix]
        private static void OnRayEnter(Tank __instance, DisplayTextManager ___displayText)
        {
            // For some reason it doesnt work on water tanks
            if (__instance.name.Equals("WaterTank")) return;

            tankInstance = __instance;
        }

        [HarmonyPatch("OnRayExit")]
        [HarmonyPostfix]
        static void OnRayExit()
        {
            tankInstance = null;
        }
    }

    [HarmonyPatch(typeof(Helper), "GetTerm")]
    internal class Patch_DisplayText
    {
        private static void Postfix(ref string __result)
        {
            if (Patch_Storage_Small.storageInstance != null && Labels.ContainsKey(Patch_Storage_Small.storageInstance.ObjectIndex) && !Labels[Patch_Storage_Small.storageInstance.ObjectIndex].IsNullOrEmpty() && __result.ToLower().Trim().Equals("open"))
                __result += "\n" + Labels[Patch_Storage_Small.storageInstance.ObjectIndex];

            if (Patch_Tank.tankInstance != null && (__result.ToLower().Trim().StartsWith("place") || __result.ToLower().Trim().StartsWith("require")) && !__result.ToLower().Contains("battery"))
                __result += "\n" + Patch_Tank.tankInstance.CurrentTankAmount + "/" + Patch_Tank.tankInstance.maxCapacity;
        }
    }

    [HarmonyPatch(typeof(SaveAndLoad), "Save")]
    static class Patch_SaveGame
    {
        static void Postfix(string filename)
        {
            string[] path = filename.Split(new char[] { '\\', '/' });
            filename = "";
            for (int i = 0; i < path.Length - 1; i++)
                filename += path[i] + "\\";
            if (filename.EndsWith(SaveAndLoad.latestStringNameEnding + "\\"))
                SaveLabels(filename + MOD_NAME + "_labeldata.json");
        }
    }

    [HarmonyPatch(typeof(LoadGameBox), "Button_LoadGame")]
    internal class Patch_LoadGame
    {
        static void Postfix() => LoadLabels();
    }

    [HarmonyPatch(typeof(NewGameBox), "Button_CreateNewGame")]
    static class Patch_NewGame
    {
        static void Postfix() => Labels = new Dictionary<uint, string>();
    }

    [HarmonyPatch(typeof(LoadSceneManager), "LoadScene")]
    static class Patch_UnloadWorld
    {
        static void Postfix(ref string sceneName)
        {
            if (sceneName == Raft_Network.MenuSceneName)
                Labels = new Dictionary<uint, string>();
        }
    }
}